package de.tools400.jako2009.webservice.datatypes.server;

import java.math.BigDecimal;
import java.text.ParseException;
import java.util.Calendar;

import de.tools400.jako2009.webservice.utils.IsoDateTimeUtil;


public class DataTypesResponse {
	
	private Boolean bool;
	private Short smallInteger;
	private Integer integer;
	private Long bigInteger;
	private Float singleFloat;
	private Double doubleFloat;
	private BigDecimal packed;
	private BigDecimal zoned;
	private String string;

	/* 
	 * 'date' ist hier als 'java.lang.String' definiert, weil Axis2 v1.5 den 
	 * Datentyp 'java.lang.Date' nicht richtig behandelt. Das umsetzen von 'Date' 
	 * in einen XML Datenstrom funktioniert. Hierbei wird ein XML String im Format 
	 * eines xs:date erzeugt.
	 * Das Erzeugen eines 'Date' Objekts aus einem XMl Datenstrom funktioniert 
	 * jedoch nicht, weil das Datum hier im Format eines xs:dateTime erwartet wird.
	 */
	private String date;

	/*
	 * 'time' ist als 'java.lang.String' definiert, weil beim Aufrufen der Methode 
	 * getTime() ein 'IllegalArgumentException' Fehler erzeugt wird.
	 */
	private String time;
	
	/*
	 * 'timestamp' ist als 'java.util.Calendar' definiert, weil dieser dem XML Schema 
	 * Typ xs:dateTime zugeordnet ist, whrend 'java.sql.Timestamp' in den Axis2
	 * Datentyp 'dateTime' umgesetzt wird.
	 * Nachteil: die Nannosekunden gehen verloren. 
	 * Vorteil : xs:dateTime ist allgemein blich. Axis2 'dateTime' ist proprietr.
	 */
	private Calendar timestamp;
	
	public Boolean getBool() {
		return bool;
	}
	public void setBool(Boolean bool) {
		this.bool = bool;
	}
	public Short getSmallInteger() {
		return smallInteger;
	}
	public void setSmallInteger(Short smallInteger) {
		this.smallInteger = smallInteger;
	}
	public Integer getInteger() {
		return integer;
	}
	public void setInteger(Integer integer) {
		this.integer = integer;
	}
	public Long getBigInteger() {
		return bigInteger;
	}
	public void setBigInteger(Long bigInteger) {
		this.bigInteger = bigInteger;
	}
	public Float getSingleFloat() {
		return singleFloat;
	}
	public void setSingleFloat(Float singleFloat) {
		this.singleFloat = singleFloat;
	}
	public Double getDoubleFloat() {
		return doubleFloat;
	}
	public void setDoubleFloat(Double doubleFloat) {
		this.doubleFloat = doubleFloat;
	}
	public BigDecimal getPacked() {
		return packed;
	}
	public void setPacked(BigDecimal packed) {
		this.packed = packed;
	}
	public BigDecimal getZoned() {
		return zoned;
	}
	public void setZoned(BigDecimal zoned) {
		this.zoned = zoned;
	}
	public String getString() {
		return string;
	}
	public void setString(String string) {
		this.string = string;
	}
	public String getDate() {
		return date;
	}
	public void setDate(String date) {
		try {
			// Sicherstellen, dass es sich um eine korrekte Datumsangabe handelt.
			IsoDateTimeUtil.parseDate(date);
		} catch (ParseException e) {
			throw new RuntimeException(e);
		}
		this.date = date;
	}
	public String getTime() {
		return time;
	}
	public void setTime(String time) {
		try {
			// Sicherstellen, dass es sich um eine korrekte Zeitangabe handelt.
			IsoDateTimeUtil.parseTime(time);
		} catch (ParseException e) {
			throw new RuntimeException(e);
		}
		this.time = time;
	}
	public Calendar getTimestamp() {
		return timestamp;
	}
	public void setTimestamp(Calendar timestamp) {
		this.timestamp = timestamp;
	}

}
