package de.tools400.jako2009.webservice.datatypes.server;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import de.tools400.jako2009.webservice.exceptions.WebServiceException;
import de.tools400.jako2009.webservice.utils.IsoDateTimeUtil;
import de.tools400.jako2009.webservice.utils.WebServiceConfiguration;

public class DataTypesService {
	
	private final Integer BOOLEAN = 1;
	private final Integer SMALL = 2;
	private final Integer INTEGER = 3;
	private final Integer BIGINT = 4;
	private final Integer FLOAT = 5;
	private final Integer DOUBLE = 6;
	private final Integer PACKED = 7;
	private final Integer ZONED = 8;
	private final Integer STRING = 9;
	private final Integer DATE = 10;
	private final Integer TIME = 11;
	private final Integer TIMESTAMP = 12;

	public DataTypesResponse echo(DataTypesRequest request) {
		DataTypesResponse response = new DataTypesResponse();
		
	    try {
	        Connection cn = getConnection();

		    CallableStatement stmt = cn.prepareCall("CALL jako2009.DataTypes_echo(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");

		    stmt.setBoolean(BOOLEAN, request.getBool());
		    stmt.setShort(SMALL, request.getSmallInteger());
		    stmt.setInt(INTEGER, request.getInteger());
		    stmt.setLong(BIGINT, request.getBigInteger());
		    stmt.setFloat(FLOAT, request.getSingleFloat());
		    stmt.setDouble(DOUBLE, request.getDoubleFloat());
		    stmt.setBigDecimal(PACKED, request.getPacked());
		    stmt.setBigDecimal(ZONED, request.getZoned());
		    stmt.setString(STRING, request.getString());
		    
			try {
				Date tmpDate = IsoDateTimeUtil.parseDate(request.getDate());
				stmt.setDate(DATE, new java.sql.Date(tmpDate.getTime()));
			} catch (ParseException e) {
				throw new RuntimeException(e);
			}

		    try {
			    stmt.setTime(TIME, new java.sql.Time(DateFormat.getTimeInstance().parse(request.getTime()).getTime()));
			} catch (ParseException e) {
				throw new RuntimeException(e);
			}
		    stmt.setTimestamp(TIMESTAMP, new Timestamp(request.getTimestamp().getTimeInMillis()));
		    
		    stmt.execute();
		    
		    ResultSet rs = stmt.getResultSet();
		    if (rs == null) {
		    	closeConnection(cn, stmt, rs);
		    	return response;
		    }
		    
		    if (rs.next() == false) {
		    	closeConnection(cn, stmt, rs);
		    	return response;
		    }

		    response.setBool(rs.getBoolean(BOOLEAN));
		    response.setSmallInteger(rs.getShort(SMALL));
		    response.setInteger(rs.getInt(INTEGER));
		    response.setBigInteger(rs.getLong(BIGINT));
		    response.setSingleFloat(rs.getFloat(FLOAT));
		    response.setDoubleFloat(rs.getDouble(DOUBLE));
		    response.setPacked(rs.getBigDecimal(PACKED));
		    response.setZoned(rs.getBigDecimal(ZONED));
		    response.setString(rs.getString(STRING));
		    response.setDate(IsoDateTimeUtil.formatDate(rs.getDate(DATE)));
		    response.setTime(DateFormat.getTimeInstance().format(rs.getTime(TIME)));
		    
		    Calendar timestamp = new GregorianCalendar();
		    timestamp.setTimeInMillis(rs.getTimestamp(TIMESTAMP).getTime());
		    response.setTimestamp(timestamp);

		    closeConnection(cn, stmt, rs);
			
		} catch (ClassNotFoundException e) {
			throw new WebServiceException(e);
		} catch (SQLException e) {
			throw new WebServiceException(e);
		}

		return response;
	}

	private Connection getConnection() throws ClassNotFoundException, SQLException {
		
		// Laden der Verbindungsdaten aus der Datei 'configuration.properties'
		String hostname = WebServiceConfiguration.theWebServiceConfiguration().hostname();
		String username = WebServiceConfiguration.theWebServiceConfiguration().username();
		String password = WebServiceConfiguration.theWebServiceConfiguration().password();

		// Laden Treiber fr DB2/400
		Class.forName("com.ibm.as400.access.AS400JDBCDriver");
		
		// Holen JDBC Connection zum System i
		Connection cn = DriverManager.getConnection("jdbc:as400://" + hostname + "/;date format=iso;time format=iso;prompt=false", username, password);
		return cn;
	}
	
	private void closeConnection(Connection cn, CallableStatement stmt, ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		
		if (stmt != null) {
			try {
				stmt.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		
		if (cn != null) {
			try {
				cn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
}
