package de.tools400.jako2009.webservice.retrievesystemstatus.server;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.sql.Types;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import de.tools400.jako2009.webservice.exceptions.WebServiceException;
import de.tools400.jako2009.webservice.utils.ArrayUtils;
import de.tools400.jako2009.webservice.utils.WebServiceConfiguration;

public class SystemStatusService {
	
	private final Integer BASIC_STATUS_SYSTEM_NAME = 1;
	private final Integer BASIC_STATUS_CURRENT_TIME = 2;
	private final Integer BASIC_STATUS_USERS_CURRENTLY_SIGNED_ON = 3;
	private final Integer BASIC_STATUS_BATCH_JOBS_WAITING_FOR_MESSAGES = 4;
	private final Integer BASIC_STATUS_BATCH_JOBS_RUNNING = 5;
	private final Integer BASIC_STATUS_BATCH_JOBS_HELD_WHILE_RUNNING = 6;
	private final Integer BASIC_STATUS_BATCH_JOBS_HELD_ON_JOB_QUEUE = 7;
	private final Integer BASIC_STATUS_BATCH_JOBS_ON_A_HELD_JOB_QUEUE = 8;
	private final Integer BASIC_STATUS_BATCH_JOBS_ON_UNASSIGNED_JOB_QUEUE = 9;

	private final Integer ADVANCED_STATUS_SYSTEM_NAME = 1;
	private final Integer ADVANCED_STATUS_CURRENT_TIME = 2;
	private final Integer ADVANCED_STATUS_ELAPSED_TIME = 3;
	private final Integer ADVANCED_STATUS_PERCENT_PROCESSING_UNIT_USED = 4;
	private final Integer ADVANCED_STATUS_JOBS_IN_SYSTEM = 5;
	private final Integer ADVANCED_STATUS_PERCENT_PERMANENT_ADDRESSES = 6;
	private final Integer ADVANCED_STATUS_PERCENT_TEMPORARY_ADDRESSES = 7;
	private final Integer ADVANCED_STATUS_SYSTEM_ASP = 8;
	private final Integer ADVANCED_STATUS_PERCENT_SYSTEM_ASP_USED = 9;
	private final Integer ADVANCED_STATUS_TOTAL_AUXILIARY_STORAGE = 10;
	private final Integer ADVANCED_STATUS_NUMBER_OF_PROCESSORS = 11;
	private final Integer ADVANCED_STATUS_ACTIVE_JOBS_IN_SYSTEM = 12;
	private final Integer ADVANCED_STATUS_MAIN_STORAGE_SIZE = 13;

	private final Integer POOL_STATUS_INFORMATION_SYSTEM_NAME = 1;
	private final Integer POOL_STATUS_INFORMATION_CURRENT_TIME = 2;
	private final Integer POOL_STATUS_INFORMATION_ELAPSED_TIME = 3;
	private final Integer POOL_STATUS_INFORMATION_SYSTEM_POOL = 4;
	private final Integer POOL_STATUS_INFORMATION_POOL_SIZE = 5;
	private final Integer POOL_STATUS_INFORMATION_RESERVED_SIZE = 6;
	private final Integer POOL_STATUS_INFORMATION_MAXIMUM_ACTIVE_THREADS = 7;
	private final Integer POOL_STATUS_INFORMATION_DATABASE_FAULTS = 8;
	private final Integer POOL_STATUS_INFORMATION_DATABASE_PAGES = 9;
	private final Integer POOL_STATUS_INFORMATION_NON_DATABASE_FAULTS = 10;
	private final Integer POOL_STATUS_INFORMATION_NON_DATABASE_PAGES = 11;
	private final Integer POOL_STATUS_INFORMATION_ACTIVE_TO_WAIT = 12;
	private final Integer POOL_STATUS_INFORMATION_WAIT_TO_INELIGIBLE = 13;
	private final Integer POOL_STATUS_INFORMATION_ACTIVE_TO_INELIGIBLE = 14;
	private final Integer POOL_STATUS_INFORMATION_POOL_NAME = 15;
	private final Integer POOL_STATUS_INFORMATION_SUBSYSTEM_NAME = 16;
	private final Integer POOL_STATUS_INFORMATION_SUBSYSTEM_LIBRARY_NAME = 17;
	private final Integer POOL_STATUS_INFORMATION_PAGING_OPTION = 18;

	private final Integer ABOUT_VERSION = 1;
	private final Integer ABOUT_COPYRIGHT = 2;
	private final Integer ABOUT_INFO = 3;

	public BasicStatusResponse retrieveBasicStatus(BasicStatusRequest request) {
		BasicStatusResponse response = new BasicStatusResponse();
		
	    try {
	        Connection cn = getConnection();

		    CallableStatement stmt = cn.prepareCall("CALL jako2009.RetrieveSystemStatus_basic(?)");
		    if (request.getReset().equals(Boolean.TRUE)) {
		        stmt.setString(1, "*YES");
		    } else {
		        stmt.setString(1, "*NO");
		    }
		    stmt.execute();
		    
		    ResultSet rs = stmt.getResultSet();
		    if (rs == null) {
		    	closeConnection(cn, stmt, rs);
		    	return response;
		    }
		    
		    if (rs.next() == false) {
		    	closeConnection(cn, stmt, rs);
		    	return response;
		    }

		    response.setSystem(rs.getString(BASIC_STATUS_SYSTEM_NAME));
		    response.setCurrentTime(rs.getTimestamp(BASIC_STATUS_CURRENT_TIME));
		    response.setUsersCurrentlySignedOn(rs.getInt(BASIC_STATUS_USERS_CURRENTLY_SIGNED_ON));
		    response.setBatchJobsWaitingForMessages(rs.getInt(BASIC_STATUS_BATCH_JOBS_WAITING_FOR_MESSAGES));
		    response.setBatchJobsRunning(rs.getInt(BASIC_STATUS_BATCH_JOBS_RUNNING));
		    response.setBatchJobsHeldWhileRunning(rs.getInt(BASIC_STATUS_BATCH_JOBS_HELD_WHILE_RUNNING));
		    response.setBatchJobsHeldOnJobQueue(rs.getInt(BASIC_STATUS_BATCH_JOBS_HELD_ON_JOB_QUEUE));
		    response.setBatchJobsOnAHeldJobQueue(rs.getInt(BASIC_STATUS_BATCH_JOBS_ON_A_HELD_JOB_QUEUE));
		    response.setBatchJobsOnUnassignedJobQueue(rs.getInt(BASIC_STATUS_BATCH_JOBS_ON_UNASSIGNED_JOB_QUEUE));

		    closeConnection(cn, stmt, rs);
			
		} catch (ClassNotFoundException e) {
			throw new WebServiceException(e);
		} catch (SQLException e) {
			throw new WebServiceException(e);
		}

		
		return response;
	}

	public AdvancedStatusResponse retrieveAdvancedStatus(AdvancedStatusRequest request) {
		AdvancedStatusResponse response = new AdvancedStatusResponse();
		
	    try {
	        Connection cn = getConnection();

		    CallableStatement stmt = cn.prepareCall("CALL jako2009.RetrieveSystemStatus_advanced(?)");
		    if (request.getReset().equals(Boolean.TRUE)) {
		        stmt.setString(1, "*YES");
		    } else {
		        stmt.setString(1, "*NO");
		    }
		    stmt.execute();
		    
		    ResultSet rs = stmt.getResultSet();
		    if (rs == null) {
		    	closeConnection(cn, stmt, rs);
		    	return response;
		    }
		    
		    if (rs.next() == false) {
		    	closeConnection(cn, stmt, rs);
		    	return response;
		    }

		    response.setSystem(rs.getString(ADVANCED_STATUS_SYSTEM_NAME));
		    
		    Timestamp tmpCurrentTime = rs.getTimestamp(ADVANCED_STATUS_CURRENT_TIME);
		    Calendar tmpCalendar = new GregorianCalendar();
		    tmpCalendar.setTimeInMillis(tmpCurrentTime.getTime());
		    response.setCurrentTime(tmpCalendar);
		    
		    Time tmpElapsedTime = rs.getTime(ADVANCED_STATUS_ELAPSED_TIME);
		    SimpleDateFormat tmpFormat = new SimpleDateFormat("HH:mm:ss");
		    response.setElapsedTime(tmpFormat.format((Date)tmpElapsedTime));
		    
		    response.setPercentProcessingUnitUsed(rs.getInt(ADVANCED_STATUS_PERCENT_PROCESSING_UNIT_USED));
		    response.setJobsInSystem(rs.getInt(ADVANCED_STATUS_JOBS_IN_SYSTEM));
		    response.setPercentPermanentAddresses(rs.getInt(ADVANCED_STATUS_PERCENT_PERMANENT_ADDRESSES));
		    response.setPercentTemporaryAddresses(rs.getInt(ADVANCED_STATUS_PERCENT_TEMPORARY_ADDRESSES));
		    response.setSystemASP(rs.getInt(ADVANCED_STATUS_SYSTEM_ASP));
		    response.setPercentSystemASPUsed(rs.getInt(ADVANCED_STATUS_PERCENT_SYSTEM_ASP_USED));
		    response.setTotalAuxiliaryStorage(rs.getInt(ADVANCED_STATUS_TOTAL_AUXILIARY_STORAGE));
		    response.setNumberOfProcessors(rs.getInt(ADVANCED_STATUS_NUMBER_OF_PROCESSORS));
		    response.setActiveJobsInSystem(rs.getInt(ADVANCED_STATUS_ACTIVE_JOBS_IN_SYSTEM));
		    response.setMainStorageSize(rs.getLong(ADVANCED_STATUS_MAIN_STORAGE_SIZE));

	    	closeConnection(cn, stmt, rs);

		} catch (ClassNotFoundException e) {
			throw new WebServiceException(e);
		} catch (SQLException e) {
			throw new WebServiceException(e);
		}

		
		return response;
	}

	public PoolStatusInformationResponse retrievePoolStatusInformation(PoolStatusInformationRequest request) {
		PoolStatusInformationResponse response = new PoolStatusInformationResponse();
		response.setSize(new Integer(0));
		
	    try {
	        Connection cn = getConnection();

		    CallableStatement stmt = cn.prepareCall("CALL jako2009.RetrieveSystemStatus_poolInformation(?)");
		    if (request.getReset().equals(Boolean.TRUE)) {
		        stmt.setString(1, "*YES");
		    } else {
		        stmt.setString(1, "*NO");
		    }
		    stmt.execute();
		    
		    ResultSet rs = stmt.getResultSet();
		    if (rs == null) {
		    	closeConnection(cn, stmt, rs);
		    	throw new WebServiceException("Stored procedure did not return any result set.");
		    }

		    int row = 0;
		    PoolStatus poolStatusArray[] = null;
		    while (rs.next()) {
		    	
		    	row++;
		    	
		    	if (poolStatusArray == null) {
		    		poolStatusArray = new PoolStatus[1];
		    	} else {
		    		poolStatusArray = (PoolStatus[]) ArrayUtils.resizeArray(poolStatusArray, row);
		    	}
		    	
		    	PoolStatus poolStatus = new PoolStatus();

			    poolStatus.setSystem(rs.getString(POOL_STATUS_INFORMATION_SYSTEM_NAME));
			    
			    Calendar tmpCurrentTime = new GregorianCalendar();
			    tmpCurrentTime.setTimeInMillis(rs.getTimestamp(POOL_STATUS_INFORMATION_CURRENT_TIME).getTime());
			    poolStatus.setCurrentTime(tmpCurrentTime);
			    
			    poolStatus.setElapsedTime(DateFormat.getTimeInstance().format(rs.getTime(POOL_STATUS_INFORMATION_ELAPSED_TIME)));
			    poolStatus.setSystemPool(rs.getInt(POOL_STATUS_INFORMATION_SYSTEM_POOL));
			    poolStatus.setPoolSize(rs.getInt(POOL_STATUS_INFORMATION_POOL_SIZE));
			    poolStatus.setReservedSize(rs.getInt(POOL_STATUS_INFORMATION_RESERVED_SIZE));
			    poolStatus.setMaximumActiveThreads(rs.getInt(POOL_STATUS_INFORMATION_MAXIMUM_ACTIVE_THREADS));
			    poolStatus.setDatabaseFaults(rs.getInt(POOL_STATUS_INFORMATION_DATABASE_FAULTS));
			    poolStatus.setDatabasePages(rs.getInt(POOL_STATUS_INFORMATION_DATABASE_PAGES));
			    poolStatus.setNondatabaseFaults(rs.getInt(POOL_STATUS_INFORMATION_NON_DATABASE_FAULTS));
			    poolStatus.setNondatabasePages(rs.getInt(POOL_STATUS_INFORMATION_NON_DATABASE_PAGES));
			    poolStatus.setActiveToWait(rs.getInt(POOL_STATUS_INFORMATION_ACTIVE_TO_WAIT));
			    poolStatus.setWaitToIneligible(rs.getInt(POOL_STATUS_INFORMATION_WAIT_TO_INELIGIBLE));
			    poolStatus.setActiveToIneligible(rs.getInt(POOL_STATUS_INFORMATION_ACTIVE_TO_INELIGIBLE));
			    poolStatus.setPoolName(rs.getString(POOL_STATUS_INFORMATION_POOL_NAME));
			    poolStatus.setSubsystemName(rs.getString(POOL_STATUS_INFORMATION_SUBSYSTEM_NAME));
			    poolStatus.setSubsystemLibraryName(rs.getString(POOL_STATUS_INFORMATION_SUBSYSTEM_LIBRARY_NAME));
			    poolStatus.setPagingOption(rs.getString(POOL_STATUS_INFORMATION_PAGING_OPTION));
		    	
			    poolStatusArray[row-1] = poolStatus;
		    }
		    
		    response.setSize(new Integer(poolStatusArray.length));
		    response.setPoolStatus(poolStatusArray);

	    	closeConnection(cn, stmt, rs);

		} catch (ClassNotFoundException e) {
			throw new WebServiceException(e);
		} catch (SQLException e) {
			throw new WebServiceException(e);
		}

		return response;
	}

	public About about() {
		About response = new About();
		
	    try {
		    Long start = Calendar.getInstance().getTimeInMillis();

	        Connection cn = getConnection();

		    CallableStatement stmt = cn.prepareCall("CALL jako2009.RetrieveSystemStatus_about(?, ?, ?)");
		    
		    stmt.registerOutParameter(ABOUT_VERSION, Types.VARCHAR);
		    stmt.registerOutParameter(ABOUT_COPYRIGHT, Types.VARCHAR);
		    stmt.registerOutParameter(ABOUT_INFO, Types.VARCHAR);
		    
		    stmt.execute();
		    
		    response.setVersion(stmt.getString(ABOUT_VERSION));
		    response.setCopyright(stmt.getString(ABOUT_COPYRIGHT));
		    response.setInfo(stmt.getString(ABOUT_INFO));
		    
	    	closeConnection(cn, stmt, null);
	    	
		    response.setExecutionTime(Calendar.getInstance().getTimeInMillis() - start);
		    
		} catch (ClassNotFoundException e) {
			throw new WebServiceException(e);
		} catch (SQLException e) {
			throw new WebServiceException(e);
		}

		return response;
	}
	
	public void testInputOnly(String aValue) {
		System.out.println("Operation 'testInputOnly()' ausgefhrt: " + aValue);
	}
	
	public String testOutputOnly() {
		System.out.println("Operation 'testOutputOnly()' ausgefhrt.");
		return "Operation 'testOutputOnly()' ausgefhrt.";
	}
	
	public String testInputOutput(String aValue) {
		System.out.println("Operation 'testInputOutput()' ausgefhrt.");
		return "Funktionswert:" + aValue;
	}

	private Connection getConnection() throws ClassNotFoundException, SQLException {
		
		// Laden der Verbindungsdaten aus der Datei 'configuration.properties'
		String hostname = WebServiceConfiguration.theWebServiceConfiguration().hostname();
		String username = WebServiceConfiguration.theWebServiceConfiguration().username();
		String password = WebServiceConfiguration.theWebServiceConfiguration().password();

		// Laden Treiber fr DB2/400
		Class.forName("com.ibm.as400.access.AS400JDBCDriver");
		
		// Holen JDBC Connection zum System i
		Connection cn = DriverManager.getConnection("jdbc:as400://" + hostname + "/;date format=iso;time format=iso;prompt=false", username, password);
		return cn;
	}
	
	private void closeConnection(Connection cn, CallableStatement stmt, ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		
		if (stmt != null) {
			try {
				stmt.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
		
		if (cn != null) {
			try {
				cn.close();
			} catch (SQLException e) {
				e.printStackTrace();
			}
		}
	}
}
